@@ -1,3 +1,4 @@
+# coding: utf-8
 """Tests for managing HTTP issues (malformed requests, etc)."""
 
 import errno
@@ -5,28 +6,57 @@ import mimetypes
 import socket
 import sys
 from unittest import mock
-
-import six
-from six.moves.http_client import HTTPConnection
-from six.moves import urllib
+import urllib.parse
+from http.client import HTTPConnection
 
 import cherrypy
-from cherrypy._cpcompat import HTTPSConnection, ntob
+from cherrypy._cpcompat import HTTPSConnection
 
 from cherrypy.test import helper
 
 
+def is_ascii(text):
+    """
+    Return True if the text encodes as ascii.
+    """
+    try:
+        text.encode('ascii')
+        return True
+    except Exception:
+        pass
+    return False
+
+
+def encode_filename(filename):
+    """
+    Given a filename to be used in a multipart/form-data,
+    encode the name. Return the key and encoded filename.
+    """
+    if is_ascii(filename):
+        return 'filename', '"{filename}"'.format(**locals())
+    encoded = urllib.parse.quote(filename, encoding='utf-8')
+    return 'filename*', "'".join((
+        'UTF-8',
+        '',  # lang
+        encoded,
+    ))
+
+
 def encode_multipart_formdata(files):
     """Return (content_type, body) ready for httplib.HTTP instance.
 
     files: a sequence of (name, filename, value) tuples for multipart uploads.
+    filename can be a string or a tuple ('filename string', 'encoding')
     """
     BOUNDARY = '________ThIs_Is_tHe_bouNdaRY_$'
     L = []
     for key, filename, value in files:
         L.append('--' + BOUNDARY)
-        L.append('Content-Disposition: form-data; name="%s"; filename="%s"' %
-                 (key, filename))
+
+        fn_key, encoded = encode_filename(filename)
+        tmpl = \
+            'Content-Disposition: form-data; name="{key}"; {fn_key}={encoded}'
+        L.append(tmpl.format(**locals()))
         ct = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
         L.append('Content-Type: %s' % ct)
         L.append('')
@@ -73,14 +103,12 @@ class HTTPTests(helper.CPWebCase):
                         count += 1
                     else:
                         if count:
-                            if six.PY3:
-                                curchar = chr(curchar)
+                            curchar = chr(curchar)
                             summary.append('%s * %d' % (curchar, count))
                         count = 1
                         curchar = c
                 if count:
-                    if six.PY3:
-                        curchar = chr(curchar)
+                    curchar = chr(curchar)
                     summary.append('%s * %d' % (curchar, count))
                 return ', '.join(summary)
 
@@ -106,7 +134,7 @@ class HTTPTests(helper.CPWebCase):
         self.body = response.fp.read()
         self.status = str(response.status)
         self.assertStatus(200)
-        self.assertBody(ntob('Hello world!'))
+        self.assertBody(b'Hello world!')
 
         # Now send a message that has no Content-Length, but does send a body.
         # Verify that CP times out the socket and responds
@@ -160,10 +188,14 @@ class HTTPTests(helper.CPWebCase):
         '''Testing that we can handle filenames with special characters. This
         was reported as a bug in:
            https://github.com/cherrypy/cherrypy/issues/1146/
-           https://github.com/cherrypy/cherrypy/issues/1397'''
+           https://github.com/cherrypy/cherrypy/issues/1397/
+           https://github.com/cherrypy/cherrypy/issues/1694/
+        '''
         # We'll upload a bunch of files with differing names.
-        fnames = ['boop.csv', 'foo, bar.csv', 'bar, xxxx.csv', 'file"name.csv',
-                  'file;name.csv', 'file; name.csv']
+        fnames = [
+            'boop.csv', 'foo, bar.csv', 'bar, xxxx.csv', 'file"name.csv',
+            'file;name.csv', 'file; name.csv', u'test_łóąä.txt',
+        ]
         for fname in fnames:
             files = [('myfile', fname, 'yunyeenyunyue')]
             content_type, body = encode_multipart_formdata(files)
@@ -189,7 +221,7 @@ class HTTPTests(helper.CPWebCase):
 
         # Test missing version in Request-Line
         c = self.make_connection()
-        c._output(ntob('GET /'))
+        c._output(b'geT /')
         c._send_output()
         if hasattr(c, 'strict'):
             response = c.response_class(c.sock, strict=c.strict, method='GET')
@@ -199,7 +231,7 @@ class HTTPTests(helper.CPWebCase):
             response = c.response_class(c.sock, method='GET')
         response.begin()
         self.assertEqual(response.status, 400)
-        self.assertEqual(response.fp.read(22), ntob('Malformed Request-Line'))
+        self.assertEqual(response.fp.read(22), b'Malformed Request-Line')
         c.close()
 
     def test_request_line_split_issue_1220(self):
@@ -221,7 +253,7 @@ class HTTPTests(helper.CPWebCase):
         c.putrequest('GET', '/')
         c.putheader('Content-Type', 'text/plain')
         # See https://github.com/cherrypy/cherrypy/issues/941
-        c._output(ntob('Re, 1.2.3.4#015#012'))
+        c._output(b're, 1.2.3.4#015#012')
         c.endheaders()
 
         response = c.getresponse()
@@ -255,14 +287,14 @@ class HTTPTests(helper.CPWebCase):
     def test_garbage_in(self):
         # Connect without SSL regardless of server.scheme
         c = HTTPConnection('%s:%s' % (self.interface(), self.PORT))
-        c._output(ntob('gjkgjklsgjklsgjkljklsg'))
+        c._output(b'gjkgjklsgjklsgjkljklsg')
         c._send_output()
         response = c.response_class(c.sock, method='GET')
         try:
             response.begin()
             self.assertEqual(response.status, 400)
             self.assertEqual(response.fp.read(22),
-                             ntob('Malformed Request-Line'))
+                             b'Malformed Request-Line')
             c.close()
         except socket.error:
             e = sys.exc_info()[1]
