@@ -19,6 +19,10 @@ from cherrypy.test import helper
 
 @pytest.fixture
 def unicode_filesystem(tmpdir):
+    _check_unicode_filesystem(tmpdir)
+
+
+def _check_unicode_filesystem(tmpdir):
     filename = tmpdir / ntou('☃', 'utf-8')
     tmpl = 'File system encoding ({encoding}) cannot support unicode filenames'
     msg = tmpl.format(encoding=sys.getfilesystemencoding())
@@ -35,7 +39,7 @@ def ensure_unicode_filesystem():
     """
     tmpdir = py.path.local(tempfile.mkdtemp())
     try:
-        unicode_filesystem(tmpdir)
+        _check_unicode_filesystem(tmpdir)
     finally:
         tmpdir.remove()
 
@@ -157,8 +161,8 @@ class StaticTest(helper.CPWebCase):
         super(cls, cls).teardown_class()
         files_to_remove = has_space_filepath, bigfile_filepath
         files_to_remove += tuple(cls.files_to_remove)
-        for f in files_to_remove:
-            f.remove_p()
+        for file in files_to_remove:
+            file.remove_p()
 
     def test_static(self):
         self.getPage('/static/index.html')
